//
// core.h
//

#ifndef CORE_H
#define CORE_H

  // data representation

  // How to represent the 4 registers?
  // Originally, there were 12 nibbles for BCD digits, 2 nibbles for
  // BCD coded exponent and 1 nibble each for the signs.
  //
  //   +-   +-    1 ,  2    3    4    5    6    7    8    9    0    1    2
  //
  //  sign sign                                                    10s  1s
  //  man  exp  \_________________ man _________________________/ \__exp__/
  //
  // | Byte 0  |  1 |  2 |  3 |  4 |  5 |  6 |  7 |  8 |  9 | 10 | 11 | 12 |
  //
  // New coding as of 2012-08-03:
  //
  // Each digit gets it dedicated byte to ease addition etc.
  // Signs are stored in one byte:  0b .... ..me  (mantissa, exponent)

#define SIGNS        0
#define M            1
#define M_HIGH       1
#define M_ONES      10
#define E           11
#define E_TENS      11
#define E_ONES      12
#define NUMBER_SIZE 13

#define NEG_M     0x01
#define NEG_E     0x02

#define X      0     //
#define Y      1     // standard stack
#define Z      2     //
#define T      3     //
#define A      4           //
#define B      5           // safe area to do math w/o affecting X or Y
#define R      6           //
#define REG_NR 7

extern char reg[REG_NR][NUMBER_SIZE];
extern char flags;
#define INFTY     0x01

void copy_reg(char* dst, char* src);
 int get_e(char* r);
void inc_e(char* r);
void shiftr1(char* r);
void shiftr(char* r, int n);
void plus(char* res, char* r1, char* r2);
void pop();
void push(char* r);
void enter();
void init_core();

#endif // CORE_H
