//
// display.c
//

#include <assert.h>
#include <curses.h>
#include "display.h"

  // "curses" stuff
  WINDOW* wnd;
  int nrows;
  int ncols;


/**
 * Initialize curses window.
 */
void init_display() {
  wnd = initscr();                // initialize curses
  cbreak();                       // send keys directly to program
  noecho();                       // don't write my typing to screen
  getmaxyx(wnd, nrows, ncols);    // check terminal dimensions
}


/**
 * Convert register to string.
 */
 // "--" "1.234567890" "12" --> "-1.234567890e-12"
 //
void rtostr(char* str, char* r) {
  if (r[SIGNS] & NEG_M) str[0] = '-';     // check for man negative
  else str[0] = ' ';

  { assert(r[M_HIGH]>=0 && r[M_HIGH]<=9); }
  str[1] = r[M_HIGH] + '0';    // access first digit of mantissa
  str[2] = '.';
  int i;                  // access digit 2-10 of mantissa
  for (i=1; i<=9; i++) str[i+2] = r[M_HIGH+i] + '0';
  str[12] = 'e';

  if (r[SIGNS] & NEG_E) str[13] = '-';    // check for exp negative
  else str[13] = ' ';

  str[14] = r[E_TENS] + '0';   // access digits of exponent
  str[15] = r[E_ONES] + '0';
  str[16] = '\0';
}

/**
 * Print values of t,z,y,x registers.
 */
void screen_refresh() {
  char str[17];   // will hold number, e.g. "-1.234567890e-12\0"
  memset(str, ' ', 16);
  str[16] = '\0';

  move(0,0);      // also possible: clear();

  printw("+------------------+\n");
  printw("| ");
  rtostr(str, reg[T]);
  printw(str);
  printw(" |\n");
  printw("| ");
  rtostr(str, reg[Z]);
  printw(str);
  printw(" |\n");
  printw("| ");
  rtostr(str, reg[Y]);
  printw(str);
  printw(" |\n");
  printw("| ");
  rtostr(str, reg[X]);
  printw(str);
  printw(" |\n");
  printw("+------------------+");

  move(nrows-1, ncols-1);       // "hide" cursor
} // screen_refresh


/**
 * Print system error instead of X register.
 */
void err_error(char* str) {
  move(4, 2);
  printw(str);
  move(nrows-1, ncols-1);       // "hide" cursor
  getch();
}


/**
 * Print message according to status of argument.
 */
void overflow(char* r) {
  if ((flags & INFTY) && (r[SIGNS] & NEG_M)) err_error(" neg infinity   ");
  else if (flags & INFTY) err_error("    infinity    ");
  else err_error(" general overflw");
}


/**
 * Close curses window.
 */
void close_display() {
  echo();
  endwin();
}
